import 'api_request_method.dart';
import 'api_resource.dart';
import 'package:cl_datahub_common/common.dart';

import 'api.dart';
import 'api_response.dart';

class ApiBase {
  final String baseUrl;
  final List<ApiResource> _resources = [];

  ApiBase(this.baseUrl);

  void bindResource(ApiResource resource) {
    if (_resources.contains(resource)) {
      throw ApiException('Cannot bind: Resource already bound to this api.');
    }

    if (resource.api != this) {
      throw ApiError('Cannot bind: Resource belongs to different api.');
    }

    _resources.add(resource);
  }

  Map<String, String> getRequestHeaders() {
    //TODO auth token and stuff
    return {};
  }

  Future<ApiResponse<TResponse>> getObject<TResponse>(String suffix,
      {Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, TResponse>(
        baseUrl, suffix, urlParams, ApiRequestMethod.GET,
        headers: getRequestHeaders(), bean: bean, query: query);
  }

  Future<ApiResponse<List<TResponse>>> getList<TResponse>(String suffix,
      {Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, List<TResponse>>(
        baseUrl, suffix, urlParams, ApiRequestMethod.GET,
        headers: getRequestHeaders(), bean: bean, query: query);
  }

  Future<ApiResponse<TResponse>> postObject<TResponse>(
      String suffix, dynamic object,
      {Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, TResponse>(
        baseUrl, suffix, urlParams, ApiRequestMethod.POST,
        headers: getRequestHeaders(), bean: bean, body: object, query: query);
  }

  Future<ApiResponse<TResponse>> putObject<TResponse>(
      String suffix, dynamic object,
      {Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, TResponse>(
        baseUrl, suffix, urlParams, ApiRequestMethod.PUT,
        headers: getRequestHeaders(), bean: bean, body: object, query: query);
  }

  Future<ApiResponse<TResponse>> patchObject<TResponse>(
      String suffix, dynamic object,
      {Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, TResponse>(
        baseUrl, suffix, urlParams, ApiRequestMethod.PATCH,
        headers: getRequestHeaders(), bean: bean, body: object, query: query);
  }

  Future<ApiResponse<TResponse>> rawRequest<TResponse>(
      ApiRequestMethod method, String suffix,
      {dynamic body,
      Map<String, dynamic>? urlParams,
      Map<String, String?>? query,
      TransferBean<TResponse>? bean}) async {
    return await request<TResponse, TResponse>(
        baseUrl, suffix, urlParams, method,
        headers: getRequestHeaders(), bean: bean, body: body, query: query);
  }

  Future<ApiResponse<void>> delete(String suffix,
      {Map<String, dynamic>? urlParams, Map<String, String?>? query}) async {
    return await request<void, void>(
        baseUrl, suffix, urlParams, ApiRequestMethod.DELETE,
        headers: getRequestHeaders(), query: query);
  }
}
