import 'package:cl_appbase/api.dart';
import 'package:cl_datahub_common/common.dart';

class ApiResource<TData> {
  final String endpointSuffix;
  final TransferBean<TData> bean;
  final ApiBase api;

  const ApiResource._(this.api, this.endpointSuffix, this.bean);
}

class ListApiResource<TData, TId> extends ApiResource<TData> {
  const ListApiResource(
      ApiBase api, String endpointSuffix, TransferBean<TData> bean)
      : super._(api, endpointSuffix, bean);

  Future<List<TData>> getList(int offset, int limit,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.getList<TData>(endpointSuffix,
        urlParams: params,
        query: {
          'offset': offset.toString(),
          'limit': limit.toString(),
          if (query != null) ...query
        },
        bean: bean);
    response.throwOnError();
    return response.data;
  }

  Future<TData> getElement(TId id,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.getObject('$endpointSuffix/$id',
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<TData> postElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.postObject(endpointSuffix, element,
        urlParams: params, query: query, bean: bean);
    response.throwOnError();
    return response.data;
  }

  Future<TData> putElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final url = element is TransferObjectBase<String> ||
            element is TransferObjectBase<int>
        ? '$endpointSuffix/${(element as TransferObjectBase).getId()}'
        : endpointSuffix;
    final response = await api.putObject(url, element,
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<TData> patchElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final url = element is TransferObjectBase<String> ||
            element is TransferObjectBase<int>
        ? '$endpointSuffix/${(element as TransferObjectBase).getId()}'
        : endpointSuffix;
    final response = await api.patchObject(url, element,
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<void> deleteElement(TId id,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.delete('$endpointSuffix/$id',
        urlParams: params, query: query);
    response.throwOnError();
  }
}

class SingleObjectApiResource<TData> extends ApiResource<TData> {
  const SingleObjectApiResource(
      ApiBase api, String endpointSuffix, TransferBean<TData> bean)
      : super._(api, endpointSuffix, bean);

  Future<TData> getElement(
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.getObject('$endpointSuffix',
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<TData> postElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.postObject(endpointSuffix, element,
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<TData> putElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.putObject(endpointSuffix, element,
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }

  Future<TData> patchElement(TData element,
      {Map<String, dynamic>? params, Map<String, String?>? query}) async {
    final response = await api.patchObject(endpointSuffix, element,
        urlParams: params, bean: bean, query: query);
    response.throwOnError();
    return response.data;
  }
}
